
# q-expansion of newform 855.2.a.j, downloaded from the LMFDB on 02 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 855
weight = 2
poly_data = [1, -3, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-2, -1, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, -1, 0], [0, 0, 0], [1, 0, 0], [-2, 2, 1], [-1, -1, -2], [-2, 0, -3], [-5, -1, 1], [1, 0, 0], [-3, -1, 1], [1, -1, 4], [3, -1, 3], [-6, -2, 1], [-5, 5, 2], [-2, -4, 1], [-1, -3, -5], [-5, 7, 1], [2, 0, -2], [1, 3, 1], [-2, 4, -2], [-4, 2, 0], [0, -4, -2], [6, -6, -4], [-1, -3, 3], [-13, -1, 2], [0, -2, 3], [0, 6, -4], [4, 4, 0], [2, 2, 4], [4, -10, -4], [-9, 7, 5], [-4, -6, 0], [-7, 7, 2], [-8, -2, -8], [10, 2, -4], [-8, 0, 4], [1, 5, -9], [-8, -4, -4], [2, 2, 9], [9, -1, 7], [11, -9, -3], [4, -6, 4], [-10, 4, -2], [5, -3, -6], [-6, 8, -5], [1, 5, 7], [-2, 6, 0], [10, 2, -8], [-4, 0, 0], [5, 3, 3], [-3, 3, 11], [-4, -2, -10], [-1, -3, -2], [6, 0, 4], [-7, 3, 4], [-5, -1, -13], [-3, 7, 7], [-5, -5, -10], [12, -16, -6], [0, 6, 14], [11, -5, -4], [-4, 4, 3], [-15, 1, -7], [-6, 10, 8], [-1, -11, -4], [-6, -6, 2], [-3, -11, -3], [17, 1, 3], [-2, 2, 3], [-3, 11, -3], [14, 4, -2], [2, 4, 4], [9, -11, -2], [-6, -4, -7], [0, -14, -5], [-23, 9, 7], [-6, 10, -2], [18, -8, 0], [4, -14, 2], [7, -11, 0], [6, 12, 12], [-15, 5, 0], [4, 10, 4], [0, 0, 10], [24, -10, -7], [-8, -8, -4], [-5, 13, -3], [-17, 13, 10], [-2, -14, -14], [-4, 4, 8], [-8, -10, 7], [-15, 11, 11], [3, 17, -4], [-2, 16, 12], [-23, 9, 6], [2, 6, 12], [5, 11, 7], [13, -1, -6], [-15, -7, -2], [10, 12, -8], [18, 8, 0], [16, -20, -2], [10, 4, -6], [-7, -5, -13], [-21, -3, -6], [22, 2, 6], [0, 0, -12], [5, -13, -9], [-10, 0, 2], [26, -20, -12], [-6, 4, 6], [10, 12, 4], [-14, 2, 2], [7, 15, 5], [-10, -2, -14], [2, 14, 4], [-15, -3, 8], [-8, 6, -11], [-3, 3, -13], [11, -1, -15], [10, 4, 12], [8, 6, -6], [10, 2, 3], [-17, 7, 3], [26, -2, 4], [-6, -10, 10], [10, -10, -2], [-3, -13, -11], [-17, 1, 2], [34, 2, 7], [-24, 4, 2], [-14, -10, 4], [30, 6, 14], [-1, -9, 1], [-14, 4, 14], [0, 18, 6], [-30, -4, 0], [-19, 17, 11], [22, -8, 0], [-1, 11, -5], [-12, -4, 12], [4, 16, -12], [-4, -10, 4], [-14, 8, 9], [27, -23, -15], [-16, -6, 6], [20, 2, 20], [-10, 4, 14], [17, -11, -13], [4, -20, -6], [38, 2, 4], [18, 10, 11], [-36, 4, -2], [-16, 0, -5], [-16, -8, -12], [22, 0, 4], [-16, -12, -12], [-32, 8, 2], [-46, 0, -2], [-32, -10, -4], [5, -11, -20], [-7, -5, 5], [15, 11, 1], [4, -20, -19], [18, -24, -4], [-5, 3, 3], [-17, -7, 13], [-24, 4, -12], [10, -10, -14]]
